"""..."""

from dataclasses import asdict

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.types import *


class Update(BaseAligo):
    """..."""

    def update_file(self, body: UpdateFileRequest) -> BaseFile:
        """..."""
        response = self._post(V2_FILE_UPDATE, body=body)
        return self._result(response, BaseFile)

    def rename_file(self, body: RenameFileRequest) -> BaseFile:
        """..."""
        return self.update_file(UpdateFileRequest(**asdict(body)))



"""..."""

from dataclasses import dataclass

from aligo.types import *
from aligo.types.Enum import *


@dataclass
class CreateFolderRequest(DataClass):
    """..."""
    name: str
    parent_file_id: str = 'root'
    drive_id: str = None
    check_name_mode: CheckNameMode = 'auto_rename'
    type: BaseFileType = None

    def __post_init__(self):
        self.type: BaseFileType = 'folder'
        super(CreateFolderRequest, self).__post_init__()



"""..."""
from typing import Iterator

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *


class Move(BaseAligo):
    """..."""

    def move_file(self, body: MoveFileRequest) -> MoveFileResponse:
        """..."""
        response = self._post(V2_FILE_MOVE, body=body)
        return self._result(response, MoveFileResponse)

    def batch_move_files(self, body: BatchMoveFilesRequest) -> Iterator[BatchSubResponse]:
        """..."""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=file_id,
                    url='/file/move',
                    body=MoveFileRequest(
                        drive_id=body.drive_id, file_id=file_id,
                        to_parent_file_id=body.to_parent_file_id,
                        overwrite=body.overwrite, auto_rename=body.auto_rename
                    )
                ) for file_id in body.file_id_list]
        ), MoveFileResponse)



"""自定义分享"""
import base64
import json
from typing import List

from aligo.core import *
from aligo.core.Create import Create
from aligo.core.File import File
from aligo.request import *
from aligo.types import *
from aligo.types.Enum import *


class CustomShare(Core):
    """..."""

    ALIGO_SHARE_SCHEMA = 'aligo://'

    @staticmethod
    def share_files_by_aligo(files: List[BaseFile]) -> str:
        """..."""
        result = []
        for file in files:
            result.append({
                'name': file.name,
                'content_hash': file.content_hash,
                'size': file.size
            })
        return CustomShare.ALIGO_SHARE_SCHEMA + base64.b64encode(json.dumps(result).encode()).decode()

    def __share_folder_by_aligo(self, parent_file_id: str, drive_id: str = None) -> List:
        """..."""
        result = []
        files = []
        # 1. 获取 parent_file_id 目录下文件列表
        for file in File.get_file_list(self, GetFileListRequest(parent_file_id=parent_file_id, drive_id=drive_id)):
            if file.type == 'file':
                # 2. 如果是文件, 添加到 files
                files.append(file)
                continue
            # 3. 否则是文件夹, 递归
            result.append([file.name, self.__share_folder_by_aligo(parent_file_id=file.file_id, drive_id=drive_id)])
        result.append(self.share_files_by_aligo(files))
        return result

    def share_folder_by_aligo(self, parent_file_id: str, drive_id: str = None) -> str:
        """..."""
        return CustomShare.ALIGO_SHARE_SCHEMA + base64.b64encode(
            json.dumps(
                self.__share_folder_by_aligo(parent_file_id=parent_file_id, drive_id=drive_id)).encode()).decode()

    def save_files_by_aligo(self, data: str, parent_file_id: str = 'root',
                            check_name_mode: CheckNameMode = 'auto_rename',
                            drive_id: str = None):
        """..."""
        if not data.startswith(self.ALIGO_SHARE_SCHEMA):
            self._auth.log.warning(f'这不是合法 aligo 分享协议: {data}')
            return
        result = []
        data = data[8:]
        data = json.loads(base64.b64decode(data))
        for obj in data:
            if isinstance(obj, list):
                # 创建文件夹
                folder = Create.create_folder(self, CreateFolderRequest(name=obj[0], parent_file_id=parent_file_id,
                                                                        drive_id=drive_id))
                x = self.save_files_by_aligo(obj[1], parent_file_id=folder.file_id)
                result.append((folder, x))
                continue
            # 否则为 str, 内含文件列表
            obj = obj[8:]
            obj = json.loads(base64.b64decode(obj))
            for file in obj:
                x = self.create_by_hash(name=file['name'], content_hash=file['content_hash'], size=file['size'],
                                        parent_file_id=parent_file_id, check_name_mode=check_name_mode,
                                        drive_id=drive_id)
                result.append(x)
        return result



"""..."""
from dataclasses import dataclass
from typing import List

from aligo.types import DataClass, AudioTranscodeTemplate


@dataclass
class GetAudioPlayInfoResponse(DataClass):
    """..."""
    template_list: List[AudioTranscodeTemplate] = None



"""..."""

from dataclasses import dataclass

from .Enum import MediaTranscodeStatus
from .MediaTransCodeTemplate import MediaTransCodeTemplate


@dataclass
class VideoTranscodeTemplate(MediaTransCodeTemplate):
    """..."""
    template_id: str = None
    status: MediaTranscodeStatus = None
    url: str = None



"""..."""

from aligo.core import *
from aligo.request import *
from aligo.response import *


class Audio(Core):
    """..."""

    def get_audio_play_info(self, file_id: str, drive_id: str = None) -> GetAudioPlayInfoResponse:
        """获取音频播放信息"""
        body = GetAudioPlayInfoRequest(file_id=file_id, drive_id=drive_id)
        return super(Audio, self).get_audio_play_info(body)



"""..."""
from dataclasses import dataclass, is_dataclass
from typing import get_type_hints, get_args, get_origin, TypeVar, Generic, Optional, List, Dict, Type

DataType = TypeVar('DataType')


@dataclass
class DataClass:
    """..."""

    def __post_init__(self):
        """序列化属性"""
        hints = get_type_hints(self)
        for k, v in hints.items():
            origin = get_origin(v)
            if origin is None:
                if is_dataclass(v):
                    setattr(self, k, _null_dict(v, getattr(self, k)))
                    continue
            for hint_type in get_args(v):
                if is_dataclass(hint_type):
                    if origin is list:
                        setattr(self, k, _null_list(hint_type, getattr(self, k)))
                    break


def _null_list(cls: Generic[DataType], may_null: Optional[List[DataType]]) -> List[DataType]:
    if may_null and len(may_null) != 0:
        if isinstance(may_null[0], dict):
            return [cls(**i) for i in may_null]
        else:
            return may_null
    else:
        return []


def _null_dict(cls: Type[DataType], may_null: Optional[Dict]) -> DataType:
    if may_null is None:
        return None
    if isinstance(may_null, dict):
        return cls(**may_null)
    return may_null



"""收藏相关"""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class Star(Core):
    """..."""
    MAX_STAR_COUNT: int = 500

    def starred_file(self,
                     file_id: str,
                     starred: bool = True,
                     drive_id: str = None) -> BaseFile:
        """收藏(或取消) 文件"""
        body = StarredFileRequest(
            file_id=file_id,
            starred=starred,
            drive_id=drive_id,
        )
        return super(Star, self).starred_file(body)

    def batch_star_files(self,
                         file_id_list: List[str],
                         starred: bool = True,
                         drive_id: str = None) -> List[BatchSubResponse]:
        """批量收藏文件"""
        body = BatchStarFilesRequest(
            drive_id=drive_id,
            file_id_list=file_id_list,
            starred=starred,
        )
        result = super(Star, self).batch_star_files(body)
        return [i for i in result]

    def get_starred_list(self, body: GetStarredListRequest = None, **kwargs) -> List[BaseFile]:
        """收藏(或取消) 文件列表"""
        if body is None:
            body = GetStarredListRequest(**kwargs)
        result = super(Star, self).get_starred_list(body)
        return [i for i in result]



"""..."""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.response import *


class Copy(Core):
    """..."""

    def copy_file(self, file_id: str = None,
                  to_parent_file_id: str = 'root',
                  new_name: str = None,
                  drive_id: str = None,
                  to_drive_id: str = None,
                  body: CopyFileRequest = None,
                  **kwargs) -> CopyFileResponse:
        """复制文件"""
        if body is None:
            body = CopyFileRequest(
                file_id=file_id,
                drive_id=drive_id,
                to_drive_id=to_drive_id,
                to_parent_file_id=to_parent_file_id,
                new_name=new_name,
                **kwargs
            )
        return super(Copy, self).copy_file(body)

    def batch_copy_files(self,
                         file_id_list: List[str] = None,
                         to_parent_file_id: str = 'root',
                         drive_id: str = None,
                         body: BatchCopyFilesRequest = None,
                         **kwargs) -> List[BatchSubResponse]:
        """批量复制"""
        if body is None:
            body = BatchCopyFilesRequest(drive_id=drive_id,
                                         file_id_list=file_id_list,
                                         to_parent_file_id=to_parent_file_id)
        result = super(Copy, self).batch_copy_files(body)
        return [i for i in result]



"""..."""
import os
import re
from typing import Iterator, List

import requests

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class Download(BaseAligo):
    """..."""
    CHUNK_SIZE = 10485760

    def get_download_url(self, body: GetDownloadUrlRequest) -> GetDownloadUrlResponse:
        """..."""
        response = self._post(V2_FILE_GET_DOWNLOAD_URL, body=body)
        return self._result(response, GetDownloadUrlResponse)

    def batch_download_url(self, body: BatchDownloadUrlRequest) -> Iterator[BatchDownloadUrlResponse]:
        """..."""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=file_id,
                    url='/file/get_download_url',
                    body=GetDownloadUrlRequest(
                        drive_id=body.drive_id, file_id=file_id
                    )
                ) for file_id in body.file_id_list]
        ), GetDownloadUrlResponse)

    @staticmethod
    def _del_special_symbol(s: str) -> str:
        """删除Windows文件名中不允许的字符"""
        return re.sub(r'[\\/:*?"<>|]', '_', s)

    def download_file(self, file_path: str, url: str) -> str:
        """下载文件"""
        file_path = os.path.abspath(file_path)
        # 递归创建目录
        os.makedirs(os.path.dirname(file_path), exist_ok=True)

        if os.path.exists(file_path):
            self._auth.log.warning(f'文件已存在,跳过下载 {file_path}')
            return file_path

        tmp_file = file_path + '.ali'

        with requests.get(url, headers={
            'referer': 'https://www.aliyundrive.com/'
        }, stream=True) as resp:
            with open(tmp_file, 'wb') as f:
                for chunk in resp.iter_content(chunk_size=int(Download.CHUNK_SIZE)):
                    f.write(chunk)

        os.renames(tmp_file, file_path)
        self._auth.log.info(f'文件下载完成 {file_path}')
        return file_path

    def download_files(self, files: List[BaseFile], local_folder: str = '.') -> List[str]:
        """批量下载文件"""
        rt = []
        for file in files:
            file_name = os.path.join(local_folder, self._del_special_symbol(file.name))
            file_path = self.download_file(file_name, file.download_url)
            rt.append(file_path)
        return rt



"""..."""
import os
from typing import List

from aligo.core import *
from aligo.core.File import File
from aligo.request import *
from aligo.response import *


class Download(Core):
    """..."""

    def get_download_url(self,
                         file_id: str,
                         file_name: str = None,
                         expire_sec: int = 14400,
                         drive_id: str = None,
                         ) -> GetDownloadUrlResponse:
        """获取下载链接"""
        body = GetDownloadUrlRequest(
            file_id=file_id,
            drive_id=drive_id,
            file_name=file_name,
            expire_sec=expire_sec,
        )
        return super(Download, self).get_download_url(body)

    def batch_download_url(self,
                           file_id_list: List[str],
                           expire_sec: int = 14400,
                           drive_id=None) -> List[BatchDownloadUrlResponse]:
        """批量获取下载链接"""
        body = BatchDownloadUrlRequest(
            drive_id=drive_id,
            file_id_list=file_id_list,
            expire_sec=expire_sec
        )
        result = super(Download, self).batch_download_url(body)
        return [i for i in result]

    def download_folder(self, folder_file_id: str, local_folder: str = '.', drive_id: str = None) -> str:
        """下载文件夹"""
        files = []
        for file in File.get_file_list(self, GetFileListRequest(parent_file_id=folder_file_id, drive_id=drive_id)):
            if file.type == 'folder':
                self.download_folder(folder_file_id=file.file_id,
                                     local_folder=os.path.join(local_folder, self._del_special_symbol(file.name)))
                continue
            files.append(file)
        self.download_files(files, local_folder=local_folder)
        return os.path.abspath(local_folder)



"""文件相关"""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class File(Core):
    """..."""

    def get_file(self,
                 file_id: str,
                 drive_id: str = None,
                 body: GetFileRequest = None,
                 **kwargs) -> BaseFile:
        """获取文件信息, 其他类中可能会用到, 所以放到基类中"""
        if body is None:
            body = GetFileRequest(
                file_id=file_id, drive_id=drive_id,
                **kwargs
            )
        return super(File, self).get_file(body)

    def get_file_list(self, parent_file_id: str = 'root', drive_id: str = None, body: GetFileListRequest = None,
                      **kwargs) -> List[BaseFile]:
        """获取文件列表"""
        if body is None:
            body = GetFileListRequest(drive_id=drive_id, parent_file_id=parent_file_id, **kwargs)
        result = super(File, self).get_file_list(body)
        return [i for i in result]

    def batch_get_files(self, file_id_list: List[str], drive_id: str = None) -> List[BatchSubResponse]:
        """..."""
        body = BatchGetFileRequest(file_id_list=file_id_list, drive_id=drive_id)
        result = super(File, self).batch_get_files(body)
        return [i for i in result]



"""创建上传相关"""
import os
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.response import *
from aligo.types import *
from aligo.types.Enum import *


class Create(Core):
    """..."""

    def create_folder(self,
                      name: str,
                      parent_file_id: str = 'root',
                      drive_id: str = None,
                      check_name_mode: CheckNameMode = 'auto_rename') -> CreateFileResponse:
        """创建文件夹"""
        body = CreateFolderRequest(
            name=name,
            parent_file_id=parent_file_id,
            drive_id=drive_id,
            check_name_mode=check_name_mode,
        )
        return super(Create, self).create_folder(body)

    def upload_files(self, file_paths: List[str], parent_file_id: str = 'root', drive_id: str = None,
                     check_name_mode: CheckNameMode = "auto_rename") -> List[BaseFile]:
        """..."""
        file_list = []
        for file_path in file_paths:
            file = self.upload_file(file_path=file_path, parent_file_id=parent_file_id, drive_id=drive_id,
                                    check_name_mode=check_name_mode)
            file_list.append(file)
        return file_list

    def upload_folder(self, folder_path: str, parent_file_id: str = 'root', drive_id: str = None,
                      check_name_mode: CheckNameMode = "auto_rename") -> List:
        """上传本地文件夹"""
        result = []
        # 1. 获取文件夹名
        folder_name = os.path.basename(folder_path)
        # 2. 在指定 parent_file_id 下创建 folder_name 文件夹, 并获取 folder BaseFile 对象
        folder = self.create_folder(folder_name, parent_file_id=parent_file_id, drive_id=drive_id,
                                    check_name_mode=check_name_mode)
        # 3. 开始扫描目标文件夹
        file: os.DirEntry
        for file in os.scandir(folder_path):
            if file.is_file():
                # 4. 如果是文件, 就上传, 并继续
                x = self.upload_file(file.path, parent_file_id=folder.file_id, name=file.name, drive_id=drive_id,
                                     check_name_mode=check_name_mode)
                result.append(x)
                continue
            # 5. 否则为文件夹, 递归
            x = self.upload_folder(folder_path=file.path, parent_file_id=folder.file_id, drive_id=drive_id,
                                   check_name_mode=check_name_mode)
            result.append({file.name: x})
        return result



"""分享文件列表响应"""

from dataclasses import dataclass, field
from typing import List

from aligo.types import DataClass, BaseShareFile


@dataclass
class GetShareFileListResponse(DataClass):
    """..."""
    items: List[BaseShareFile] = field(default_factory=list)
    next_marker: str = ''



"""创建文件夹, 上传文件等"""
import hashlib
import math
import os
from dataclasses import asdict
from typing import Union

import requests
from requests.adapters import HTTPAdapter

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *
from aligo.types.Enum import *


class Create(BaseAligo):
    """创建文件: 1.创建文件 2.上传文件 3.下载文件"""

    CHUNK_SIZE: int = 10485760

    def create_file(self, body: CreateFileRequest) -> CreateFileResponse:
        """创建文件, 可用于上传文件"""
        response = self._post(V2_FILE_CREATE, body=body)
        return self._result(response, CreateFileResponse, status_code=201)

    def create_folder(self, body: CreateFolderRequest) -> CreateFileResponse:
        """..."""
        return self.create_file(CreateFileRequest(**asdict(body)))

    def complete_file(self, body: CompleteFileRequest) -> BaseFile:
        """当文件上传完成时调用"""
        response = self._post(V2_FILE_COMPLETE, body=body)
        return self._result(response, BaseFile)

    @staticmethod
    def _get_part_info_list(file_size: int):
        """根据文件大小, 返回 part_info_list """
        # 以10MB为一块: 10485760
        return [UploadPartInfo(part_number=i) for i in range(1, math.ceil(file_size / Create.CHUNK_SIZE) + 1)]

    def _pre_hash(self, file_path: str, file_size: int, name: str, parent_file_id='root', drive_id=None,
                  check_name_mode: CheckNameMode = 'auto_rename') -> CreateFileResponse:
        with open(file_path, 'rb') as f:
            pre_hash = hashlib.sha1(f.read(1024)).hexdigest()
        body = CreateFileRequest(
            drive_id=drive_id,
            part_info_list=self._get_part_info_list(file_size),
            parent_file_id=parent_file_id,
            name=name,
            type='file',
            check_name_mode=check_name_mode,
            size=file_size,
            pre_hash=pre_hash
        )
        response = self._post(V2_FILE_CREATE, body=body)
        part_info = self._result(response, CreateFileResponse, [201, 409])
        return part_info

    def _content_hash(self, file_path: str, file_size: int, name: str, parent_file_id='root', drive_id=None,
                      check_name_mode: CheckNameMode = 'auto_rename') -> CreateFileResponse:
        with open(file_path, 'rb') as f:
            content_hash = hashlib.sha1(f.read()).hexdigest().upper()
        body = CreateFileRequest(
            drive_id=drive_id,
            part_info_list=self._get_part_info_list(file_size),
            parent_file_id=parent_file_id,
            name=name,
            type='file',
            check_name_mode=check_name_mode,
            size=file_size,
            content_hash=content_hash,
            content_hash_name="sha1"
        )
        response = self._post(V2_FILE_CREATE, body=body)
        part_info = self._result(response, CreateFileResponse, 201)
        return part_info

    def _put_data(self, file_path: str, part_info: CreateFileResponse) -> Union[BaseFile, Null]:
        """上传数据"""
        with open(file_path, 'rb') as f:
            for i in part_info.part_info_list:
                # 不能使用 self._session.put
                ss = requests.session()
                # ss.mount('http://', HTTPAdapter(max_retries=5))
                ss.mount('https://', HTTPAdapter(max_retries=5))
                ss.put(data=f.read(Create.CHUNK_SIZE), url=i.upload_url)
                # response = requests.put(data=f.read(Create.CHUNK_SIZE), url=i.upload_url)
                # print(response)
        # complete
        return self.complete_file(CompleteFileRequest(
            drive_id=part_info.drive_id,
            file_id=part_info.file_id,
            upload_id=part_info.upload_id,
            part_info_list=part_info.part_info_list
        ))

    def upload_file(
            self,
            file_path: str,
            parent_file_id: str = 'root',
            name: str = None,
            drive_id: str = None,
            check_name_mode: CheckNameMode = "auto_rename"
    ) -> Union[BaseFile, CreateFileResponse]:
        """..."""
        self._auth.log.info(f'开始上传文件 {file_path}')

        if name is None:
            name = os.path.basename(file_path)

        if drive_id is None:
            drive_id = self.default_drive_id

        file_size = os.path.getsize(file_path)
        if file_size > 1024:  # 1kB
            # 1. pre_hash
            part_info = self._content_hash(file_path=file_path, file_size=file_size, name=name,
                                           parent_file_id=parent_file_id, drive_id=drive_id,
                                           check_name_mode=check_name_mode)
            # exists=True
            if part_info.exist:
                self._auth.log.warning(f'文件已存在, 跳过 {file_path} {part_info.file_id}')
                # return self.get_file(GetFileRequest(file_id=part_info.file_id))
                return part_info

            if part_info.code == 'PreHashMatched':
                # 2. content_hash
                part_info = self._content_hash(file_path=file_path, file_size=file_size, name=name,
                                               parent_file_id=parent_file_id, drive_id=drive_id,
                                               check_name_mode=check_name_mode)
                if part_info.rapid_upload:
                    # return self.get_file(GetFileRequest(file_id=part_info.file_id))
                    return part_info
            # 开始上传
            return self._put_data(file_path, part_info)

        # 2. content_hash
        part_info = self._content_hash(file_path=file_path, file_size=file_size, name=name,
                                       parent_file_id=parent_file_id, drive_id=drive_id,
                                       check_name_mode=check_name_mode)
        if part_info.rapid_upload:
            # return self.get_file(GetFileRequest(file_id=part_info.file_id))
            return part_info
        # 开始上传
        return self._put_data(file_path, part_info)

    def create_by_hash(
            self,
            name: str,
            content_hash: str,
            size: int,
            parent_file_id: str = 'root',
            check_name_mode: CheckNameMode = 'auto_rename',
            drive_id: str = None
    ) -> CreateFileResponse:
        """..."""
        self._auth.log.info(f'开始秒传 {name} {content_hash} {size}')
        body = CreateFileRequest(
            name=name,
            content_hash=content_hash,
            size=size,
            parent_file_id=parent_file_id,
            drive_id=drive_id,
            type='file',
            content_hash_name='sha1',
            check_name_mode=check_name_mode
        )
        return self.create_file(body)

    # def batch_create_by_hash(self, ) -> List[BaseFile]:
    #     """无法实现"""



"""..."""
from typing import Iterator, Union

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class File(BaseAligo):
    """..."""

    def get_file_list(self, body: GetFileListRequest) -> Iterator[BaseFile]:
        """..."""
        yield from self._list_file(V2_FILE_LIST, body, GetFileListResponse)

    def get_file_by_path(self, path: str = '/', parent_file_id: str = 'root',
                         drive_id: str = None) -> Union[BaseFile, None]:
        """成功则返回一个BaseFile对象, 失败返回None"""
        path = path.strip('/')
        if len(path) == 0:
            if parent_file_id == 'root':
                return None
            else:
                return self.get_file(GetFileRequest(file_id=parent_file_id, drive_id=drive_id))

        file = None
        for name in path.split('/'):
            file_list = File.get_file_list(self, GetFileListRequest(parent_file_id=parent_file_id, drive_id=drive_id))
            find: bool = False
            for file in file_list:
                if file.name == name:
                    find = True
                    parent_file_id = file.file_id
                    break
            if not find:
                return None
        return file

    def batch_get_files(self, body: BatchGetFileRequest) -> Iterator[BatchSubResponse]:
        """..."""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=file_id,
                    url='/file/get',
                    body=GetFileRequest(
                        drive_id=body.drive_id, file_id=file_id
                    )
                ) for file_id in body.file_id_list]
        ), GetFileRequest)



"""..."""

from aligo.core import *
from aligo.core.Config import *
from aligo.types import *


class User(BaseAligo):
    """..."""

    def get_user(self, f5: bool = False) -> BaseUser:
        """..."""
        if self._user is None or f5:
            response = self._post(V2_USER_GET)
            # response.status_code == 200 or self._error_log_exit(response)
            # self._user = BaseUser(**response.json())
            self._user = self._result(response, BaseUser)
        return self._user



"""..."""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.types import *
from aligo.types.Enum import *


class Update(Core):
    """..."""

    def rename_file(self,
                    name: str,
                    file_id: str,
                    check_name_mode: CheckNameMode = 'refuse',
                    drive_id: str = None) -> BaseFile:
        """重命名文件(夹)"""
        body = RenameFileRequest(
            name=name,
            file_id=file_id,
            check_name_mode=check_name_mode,
            drive_id=drive_id,
        )
        return super(Update, self).rename_file(body)

    def batch_rename_files(
            self,
            file_id_list: List[str],
            new_name_list: List[str],
            check_name_mode: CheckNameMode = 'refuse',
            drive_id: str = None) -> List[BaseFile]:
        """..."""
        if drive_id is None:
            drive_id = self.default_drive_id

        ids_len = len(file_id_list)
        if ids_len != len(new_name_list):
            self._auth.log.warning(f'长度不一致 file_id_list{ids_len}, new_name_list {len(new_name_list)}')
            return

        result = []
        for i in self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=file_id_list[j],
                    url='/file/update',
                    body=RenameFileRequest(
                        name=new_name_list[j],
                        file_id=file_id_list[j],
                        drive_id=drive_id,
                        check_name_mode=check_name_mode
                    )
                ) for j in range(ids_len)]
        ), BaseFile):
            result.append(i)

        return result



"""..."""
from dataclasses import dataclass

from .DataClass import DataClass


@dataclass
class VideoMediaAudioStream(DataClass):
    """..."""
    code_name: str = None
    duration: str = None
    bit_rate: str = None
    channel_layout: str = None
    channels: int = None
    sample_rate: str = None



"""..."""
from dataclasses import dataclass

from aligo.types import *
from aligo.types.Enum import *


@dataclass
class GetShareFileRequest(DataClass):
    """..."""
    share_id: str = None
    fields: GetFileFields = '*'
    file_id: str = None
    image_thumbnail_process: str = 'image/resize,w_1920/format,jpeg'
    image_url_process: str = 'image/resize,w_375/format,jpeg'
    url_expire_sec: str = 14400
    video_thumbnail_process: str = 'video/snapshot,t_1000,f_jpg,ar_auto,w_375'



"""..."""
from dataclasses import dataclass

from aligo.types import DataClass


@dataclass
class GetShareTokenResponse(DataClass):
    """..."""
    share_token: str = None
    expire_time: str = None
    expires_in: int = None



"""批量响应"""

from dataclasses import dataclass

from aligo.types import *
from aligo.types.DataClass import DataType


@dataclass
class BatchSubResponse(DataClass):
    """..."""
    id: str = None
    status: int = None
    body: DataType = None
    method: str = None



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DataClass
from .BatchSubRequest import BatchSubRequest


@dataclass
class BatchRequest(DataClass):
    """..."""
    requests: List[BatchSubRequest] = field(default_factory=list)
    resource: str = 'file'



"""..."""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class Recyclebin(Core):
    """删除文件太过危险, 只提供移动文件到回收站的功能"""

    def move_file_to_trash(self, file_id: str, drive_id: str = None) -> MoveFileToTrashResponse:
        """移动文件到回收站"""
        body = MoveFileToTrashRequest(file_id=file_id, drive_id=drive_id)
        return super(Recyclebin, self).move_file_to_trash(body)

    def batch_move_to_trash(self, file_id_list: List[str], drive_id: str = None) -> List[BatchSubResponse]:
        """批量移动到回收站"""
        body = BatchMoveToTrashRequest(drive_id=drive_id, file_id_list=file_id_list)
        result = super(Recyclebin, self).batch_move_to_trash(body)
        return [i for i in result]

    def restore_file(self, file_id: str, drive_id: str = None) -> RestoreFileResponse:
        """恢复文件"""
        body = RestoreFileRequest(drive_id=drive_id, file_id=file_id)
        return super(Recyclebin, self).restore_file(body)

    def batch_restore_files(self, file_id_list: List[str], drive_id: str = None) -> List[BatchSubResponse]:
        """批量恢复文件"""
        body = BatchRestoreRequest(drive_id=drive_id, file_id_list=file_id_list)
        result = super(Recyclebin, self).batch_restore_files(body)
        return [i for i in result]

    def get_recyclebin_list(self, body: GetRecycleBinListRequest = None, **kwargs) -> List[BaseFile]:
        """获取回收站文件列表"""
        if body is None:
            body = GetRecycleBinListRequest(**kwargs)
        result = super(Recyclebin, self).get_recyclebin_list(body)
        return [i for i in result]



"""..."""
# 必须在头部, 不能使用Pycharm的自动格式化
# from .Enum import *

from .AudioMeta import AudioMeta
from .AudioMusicMeta import AudioMusicMeta
from .AudioTranscodeTemplate import AudioTranscodeTemplate
from .BaseDrive import BaseDrive
from .BaseFile import BaseFile
from .BaseShareFile import BaseShareFile
from .BaseUser import BaseUser
from .ClientInfo import ClientInfo
from .CroppingBoundary import CroppingBoundary
from .CroppingSuggestionItem import CroppingSuggestionItem
from .DataClass import DataClass
from .ImageMedia import ImageMedia
from .ImageQuality import ImageQuality
from .ImageTag import ImageTag
from .MediaTransCodeTemplate import MediaTransCodeTemplate
from .Null import Null
from .PersonalRightsInfo import PersonalRightsInfo
from .PersonalSpaceInfo import PersonalSpaceInfo
from .Privilege import Privilege
from .RateLimit import RateLimit
from .ShareLinkBaseFile import ShareLinkBaseFile
from .ShareLinkSchema import ShareLinkSchema
from .SystemTag import SystemTag
from .Token import Token
from .UploadPartInfo import UploadPartInfo
from .VideoMedia import VideoMedia
from .VideoMediaAudioStream import VideoMediaAudioStream
from .VideoMediaVideoStream import VideoMediaVideoStream
from .VideoPreview import VideoPreview
from .VideoPreviewSprite import VideoPreviewSprite
from .VideoTranscodeTemplate import VideoTranscodeTemplate



"""更新分享链接响应"""

from dataclasses import dataclass, field
from typing import List

from aligo.types import ShareLinkSchema


@dataclass
class UpdateShareLinkResponse(ShareLinkSchema):
    """更新分享链接响应"""
    file_path_list: List[str] = field(default_factory=list)



"""..."""
import requests


class Null:
    """表示一个失败的结果"""

    def __init__(self, response: requests.Response):
        self.response = response

    def __repr__(self):
        return self.response.text

    def __bool__(self):
        return False



"""..."""
import json
import traceback
from dataclasses import asdict
from typing import Generic, List, Iterator, Dict, Callable
from typing import Union

import requests

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *
from aligo.types.DataClass import DataType
from aligo.types.Enum import *


class BaseAligo:
    """..."""

    def __init__(self, auth: Optional[Auth] = None):
        self._auth: Auth = auth or Auth()
        self._session: requests.Session = self._auth.session
        self._token: Token = self._auth.token
        self._user: Optional[BaseUser] = None
        self._personal_info: Optional[GetPersonalInfoResponse] = None
        self._default_drive: Optional[BaseDrive] = None

    def _post(self, path: str, host: str = API_HOST, body: Union[DataType, Dict] = None) -> requests.Response:
        """统一处理数据类型和 drive_id"""
        if body is None:
            body = {}
        elif isinstance(body, DataClass):
            body = asdict(body)

        if 'drive_id' in body and body['drive_id'] is None:
            # 如果存在 attr drive_id 并且它是 None，并将 default_drive_id 设置为它
            body['drive_id'] = self.default_drive_id

        return self._auth.post(path=path, host=host, body=body)

    @property
    def default_drive_id(self):
        """默认 drive_id"""
        return self._token.default_drive_id

    @property
    def default_sbox_drive_id(self):
        """默认保险箱 drive_id"""
        return self._token.default_sbox_drive_id

    @property
    def user_name(self):
        """用户名"""
        return self._token.user_name

    @property
    def user_id(self):
        """用户 id"""
        return self._token.user_id

    @property
    def nick_name(self):
        """昵称"""
        return self._token.nick_name

    def _result(self, response: requests.Response,
                dcls: Generic[DataType],
                status_code: Union[List, int] = 200) -> Union[Null, DataType]:
        """统一处理响应

        :param response:
        :param dcls:
        :param status_code:
        :return:
        """
        if isinstance(status_code, int):
            status_code = [status_code]
        if response.status_code in status_code:
            text = response.text
            if not text.startswith('{'):
                return dcls()
            try:
                return dcls(**json.loads(text))
            except TypeError:
                self._auth.debug_log(response)
                traceback.print_exc()
        self._auth.log.warning(f'{response.status_code} {response.text}')
        return Null(response)

    def _list_file(self, PATH: str, body: DataClass, ResponseType: Callable) -> Iterator[DataType]:
        """枚举文件: 用于统一处理 1.文件列表 2.搜索文件列表 3.收藏列表 4.回收站列表"""
        response = self._post(PATH, body=body)
        file_list = self._result(response, ResponseType)
        if isinstance(file_list, Null):
            yield file_list
            return
        for item in file_list.items:
            yield item
        if file_list.next_marker != '':
            body.marker = file_list.next_marker
            yield from self._list_file(PATH=PATH, body=body, ResponseType=ResponseType)

    def get_file(self, body: GetFileRequest) -> BaseFile:
        """获取文件信息, 其他类中可能会用到, 所以放到基类中"""
        response = self._post(V2_FILE_GET, body=body)
        return self._result(response, BaseFile)

    def get_personal_info(self) -> GetPersonalInfoResponse:
        """..."""
        response = self._post(V2_DATABOX_GET_PERSONAL_INFO)
        return self._result(response, GetPersonalInfoResponse)

    BATCH_COUNT = 100

    @staticmethod
    def _list_split(ll: List[DataType], n: int) -> List[List[DataType]]:
        rt = []
        for i in range(0, len(ll), n):
            rt.append(ll[i:i + n])
        return rt

    def batch_request(self, body: BatchRequest, body_type: DataType):
        """..."""
        for request_list in self._list_split(body.requests, self.BATCH_COUNT):
            response = self._post(V2_BATCH, body={
                "requests": [
                    {
                        "body": asdict(request.body),
                        "headers": request.headers,
                        "id": request.id,
                        "method": request.method,
                        "url": request.url
                    } for request in request_list
                ],
                "resource": body.resource
            })

            if response.status_code != 200:
                yield Null(response)
                return

            for batch in response.json()['responses']:
                i = BatchSubResponse(**batch)
                if i.body:
                    try:
                        # 不是都会成功
                        # eg: {'code': 'AlreadyExist.File', 'message': "The resource file has already exists. drive has the same file, can't update, file_id 609887cca951bf4feca54c6ebd0a91a03b826949"}
                        # status 409
                        i.body = body_type(**i.body)
                    except TypeError:
                        # self._auth.log.warning(i)
                        pass
                yield i
